@@ -7,16 +7,31 @@
 # For further information on the license, see the LICENSE.txt file        #
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
-# pylint: disable=too-many-arguments,wrong-import-position
-"""The `verdi` command line interface."""
-import click_completion
+"""Sub commands of the ``verdi`` command line interface.
 
-# Activate the completion of parameter types provided by the click_completion package
-click_completion.init()
-
-# Import to populate the `verdi` sub commands
+The commands need to be imported here for them to be registered with the top-level command group.
+"""
 from aiida.cmdline.commands import (
-    cmd_archive, cmd_calcjob, cmd_code, cmd_comment, cmd_completioncommand, cmd_computer, cmd_config, cmd_data,
-    cmd_database, cmd_daemon, cmd_devel, cmd_export, cmd_graph, cmd_group, cmd_help, cmd_import, cmd_node, cmd_plugin,
-    cmd_process, cmd_profile, cmd_rehash, cmd_restapi, cmd_run, cmd_setup, cmd_shell, cmd_status, cmd_user
+    cmd_archive,
+    cmd_calcjob,
+    cmd_code,
+    cmd_computer,
+    cmd_config,
+    cmd_daemon,
+    cmd_data,
+    cmd_database,
+    cmd_devel,
+    cmd_group,
+    cmd_help,
+    cmd_node,
+    cmd_plugin,
+    cmd_process,
+    cmd_profile,
+    cmd_restapi,
+    cmd_run,
+    cmd_setup,
+    cmd_shell,
+    cmd_status,
+    cmd_storage,
+    cmd_user,
 )
