@@ -8,21 +8,18 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """`verdi node` command."""
-
-import logging
-import shutil
 import pathlib
+import shutil
 
 import click
 import tabulate
 
 from aiida.cmdline.commands.cmd_verdi import verdi
-from aiida.cmdline.params import options, arguments
+from aiida.cmdline.params import arguments, options
 from aiida.cmdline.params.types.plugin import PluginParamType
 from aiida.cmdline.utils import decorators, echo, multi_line_input
 from aiida.cmdline.utils.decorators import with_dbenv
-from aiida.common import exceptions
-from aiida.common import timezone
+from aiida.common import exceptions, timezone
 from aiida.common.links import GraphTraversalRules
 
 
@@ -42,12 +39,12 @@ def verdi_node_repo():
 @with_dbenv()
 def repo_cat(node, relative_path):
     """Output the content of a file in the node repository folder."""
+    import errno
     from shutil import copyfileobj
     import sys
-    import errno
 
     try:
-        with node.open(relative_path, mode='rb') as fhandle:
+        with node.base.repository.open(relative_path, mode='rb') as fhandle:
             copyfileobj(fhandle, sys.stdout.buffer)
     except OSError as exception:
         # The sepcial case is breakon pipe error, which is usually OK.
@@ -58,7 +55,7 @@ def repo_cat(node, relative_path):
 
 @verdi_node_repo.command('ls')
 @arguments.NODE()
-@click.argument('relative_path', type=str, default='.')
+@click.argument('relative_path', type=str, required=False)
 @click.option('-c', '--color', 'color', flag_value=True, help='Use different color for folders and files.')
 @with_dbenv()
 def repo_ls(node, relative_path, color):
@@ -99,7 +96,7 @@ def repo_dump(node, output_directory):
         Recursively copy the content at the ``key`` path in the given node to
         the ``output_dir``.
         """
-        for file in node.list_objects(key):
+        for file in node.base.repository.list_objects(key):
             # Not using os.path.join here, because this is the "path"
             # in the AiiDA node, not an actual OS - level path.
             file_key = file.name if not key else f'{key}/{file.name}'
@@ -113,7 +110,7 @@ def repo_dump(node, output_directory):
                 assert file.file_type == FileType.FILE
                 out_file_path = output_dir / file.name
                 assert not out_file_path.exists()
-                with node.open(file_key, 'rb') as in_file:
+                with node.base.repository.open(file_key, 'rb') as in_file:
                     with out_file_path.open('wb') as out_file:
                         shutil.copyfileobj(in_file, out_file)
 
@@ -203,9 +200,9 @@ def node_show(nodes, print_groups):
         echo.echo(get_node_info(node))
 
         if print_groups:
-            from aiida.orm.querybuilder import QueryBuilder
-            from aiida.orm.groups import Group
             from aiida.orm import Node  # pylint: disable=redefined-outer-name
+            from aiida.orm.groups import Group
+            from aiida.orm.querybuilder import QueryBuilder
 
             # pylint: disable=invalid-name
             qb = QueryBuilder()
@@ -237,10 +234,10 @@ def echo_node_dict(nodes, keys, fmt, identifier, raw, use_attrs=True):
             id_value = node.uuid
 
         if use_attrs:
-            node_dict = node.attributes
+            node_dict = node.base.attributes.all
             dict_name = 'attributes'
         else:
-            node_dict = node.extras
+            node_dict = node.base.extras.all
             dict_name = 'extras'
 
         if keys is not None:
@@ -279,43 +276,21 @@ def extras(nodes, keys, fmt, identifier, raw):
     echo_node_dict(nodes, keys, fmt, identifier, raw, use_attrs=False)
 
 
-@verdi_node.command()
-@arguments.NODES()
-@click.option('-d', '--depth', 'depth', default=1, help='Show children of nodes up to given depth')
-@with_dbenv()
-@decorators.deprecated_command('This command will be removed in `aiida-core==2.0.0`.')
-def tree(nodes, depth):
-    """Show a tree of nodes starting from a given node."""
-    from aiida.common import LinkType
-    from aiida.cmdline.utils.ascii_vis import NodeTreePrinter
-
-    for node in nodes:
-        NodeTreePrinter.print_node_tree(node, depth, tuple(LinkType.__members__.values()))
-
-        if len(nodes) > 1:
-            echo.echo('')
-
-
 @verdi_node.command('delete')
 @click.argument('identifier', nargs=-1, metavar='NODES')
-@options.VERBOSE()
 @options.DRY_RUN()
 @options.FORCE()
 @options.graph_traversal_rules(GraphTraversalRules.DELETE.value)
 @with_dbenv()
-def node_delete(identifier, dry_run, verbose, force, **traversal_rules):
+def node_delete(identifier, dry_run, force, **traversal_rules):
     """Delete nodes from the provenance graph.
 
     This will not only delete the nodes explicitly provided via the command line, but will also include
     the nodes necessary to keep a consistent graph, according to the rules outlined in the documentation.
     You can modify some of those rules using options of this command.
     """
-    from aiida.common.log import override_log_formatter_context
     from aiida.orm.utils.loaders import NodeEntityLoader
-    from aiida.tools import delete_nodes, DELETE_LOGGER
-
-    verbosity = logging.DEBUG if verbose else logging.INFO
-    DELETE_LOGGER.setLevel(verbosity)
+    from aiida.tools import delete_nodes
 
     pks = []
 
@@ -332,8 +307,7 @@ def node_delete(identifier, dry_run, verbose, force, **traversal_rules):
         echo.echo_warning(f'YOU ARE ABOUT TO DELETE {len(pks)} NODES! THIS CANNOT BE UNDONE!')
         return not click.confirm('Shall I continue?', abort=True)
 
-    with override_log_formatter_context('%(message)s'):
-        _, was_deleted = delete_nodes(pks, dry_run=dry_run or _dry_run_callback, **traversal_rules)
+    _, was_deleted = delete_nodes(pks, dry_run=dry_run or _dry_run_callback, **traversal_rules)
 
     if was_deleted:
         echo.echo_success('Finished deletion.')
@@ -389,7 +363,7 @@ def rehash(nodes, entry_point, force):
 
     with click.progressbar(to_hash, label='Rehashing Nodes:') as iter_hash:
         for node, in iter_hash:
-            node.rehash()
+            node.base.caching.rehash()
 
     echo.echo_success(f'{num_nodes} nodes re-hashed.')
 
@@ -432,7 +406,6 @@ def verdi_graph():
 )
 @click.option('-o', '--process-out', is_flag=True, help='Show outgoing links for all processes.')
 @click.option('-i', '--process-in', is_flag=True, help='Show incoming links for all processes.')
-@options.VERBOSE(help='Print verbose information of the graph traversal.')
 @click.option(
     '-e',
     '--engine',
@@ -453,15 +426,14 @@ def verdi_graph():
 @click.option('-s', '--show', is_flag=True, help='Open the rendered result with the default application.')
 @decorators.with_dbenv()
 def graph_generate(
-    root_node, link_types, identifier, ancestor_depth, descendant_depth, process_out, process_in, engine, verbose,
-    output_format, highlight_classes, show
+    root_node, link_types, identifier, ancestor_depth, descendant_depth, process_out, process_in, engine, output_format,
+    highlight_classes, show
 ):
     """
     Generate a graph from a ROOT_NODE (specified by pk or uuid).
     """
     # pylint: disable=too-many-arguments
     from aiida.tools.visualization import Graph
-    print_func = echo.echo_info if verbose else None
     link_types = {'all': (), 'logic': ('input_work', 'return'), 'data': ('input_calc', 'create')}[link_types]
 
     echo.echo_info(f'Initiating graphviz engine: {engine}')
@@ -475,7 +447,6 @@ def graph_generate(
         annotate_links='both',
         include_process_outputs=process_out,
         highlight_classes=highlight_classes,
-        print_func=print_func
     )
     echo.echo_info(f'Recursing descendants, max depth={descendant_depth}')
     graph.recurse_descendants(
@@ -485,7 +456,6 @@ def graph_generate(
         annotate_links='both',
         include_process_inputs=process_in,
         highlight_classes=highlight_classes,
-        print_func=print_func
     )
     output_file_name = graph.graphviz.render(
         filename=f'{root_node.pk}.{engine}', format=output_format, view=show, cleanup=True
@@ -509,7 +479,7 @@ def comment_add(nodes, content):
         content = multi_line_input.edit_comment()
 
     for node in nodes:
-        node.add_comment(content)
+        node.base.comments.add(content)
 
     echo.echo_success(f'comment added to {len(nodes)} nodes')
 
@@ -523,7 +493,7 @@ def comment_update(comment_id, content):
     from aiida.orm.comments import Comment
 
     try:
-        comment = Comment.objects.get(id=comment_id)
+        comment = Comment.collection.get(id=comment_id)
     except (exceptions.NotExistent, exceptions.MultipleObjectsError):
         echo.echo_critical(f'comment<{comment_id}> not found')
 
@@ -547,7 +517,7 @@ def comment_show(user, nodes):
         echo.echo(msg)
         echo.echo('*' * len(msg))
 
-        all_comments = node.get_comments()
+        all_comments = node.base.comments.all()
 
         if user is not None:
             comments = [comment for comment in all_comments if comment.user.email == user.email]
@@ -555,14 +525,14 @@ def comment_show(user, nodes):
             if not comments:
                 valid_users = ', '.join(set(comment.user.email for comment in all_comments))
                 echo.echo_warning(f'no comments found for user {user}')
-                echo.echo_info(f'valid users found for Node<{node.pk}>: {valid_users}')
+                echo.echo_report(f'valid users found for Node<{node.pk}>: {valid_users}')
 
         else:
             comments = all_comments
 
         for comment in comments:
             comment_msg = [
-                f'Comment<{comment.id}> for Node<{node.pk}> by {comment.user.email}',
+                f'Comment<{comment.pk}> for Node<{node.pk}> by {comment.user.email}',
                 f"Created on {timezone.localtime(comment.ctime).strftime('%Y-%m-%d %H:%M')}",
                 f"Last modified on {timezone.localtime(comment.mtime).strftime('%Y-%m-%d %H:%M')}",
                 f'\n{comment.content}\n',
@@ -570,7 +540,7 @@ def comment_show(user, nodes):
             echo.echo('\n'.join(comment_msg))
 
         if not comments:
-            echo.echo_info('no comments found')
+            echo.echo_report('no comments found')
 
 
 @verdi_comment.command('remove')
@@ -585,7 +555,7 @@ def comment_remove(force, comment):
         click.confirm(f'Are you sure you want to remove comment<{comment}>', abort=True)
 
     try:
-        Comment.objects.delete(comment)
+        Comment.collection.delete(comment)
     except exceptions.NotExistent as exception:
         echo.echo_critical(f'failed to remove comment<{comment}>: {exception}')
     else:
