@@ -16,7 +16,7 @@ profiles can be selected at hook-up (-p flag).
 import click
 
 from aiida.cmdline.commands.cmd_verdi import verdi
-from aiida.cmdline.params.options import HOSTNAME, PORT, DEBUG
+from aiida.cmdline.params.options import DEBUG, HOSTNAME, PORT
 from aiida.restapi.common import config
 
 
@@ -37,7 +37,6 @@ from aiida.restapi.common import config
     default=config.CLI_DEFAULTS['WSGI_PROFILE'],
     help='Whether to enable WSGI profiler middleware for finding bottlenecks'
 )
-@click.option('--hookup/--no-hookup', 'hookup', is_flag=True, default=None, help='Hookup app to flask server')
 @click.option(
     '--posting/--no-posting',
     'posting',
@@ -46,7 +45,7 @@ from aiida.restapi.common import config
     help='Enable POST endpoints (currently only /querybuilder).',
     hidden=True,
 )
-def restapi(hostname, port, config_dir, debug, wsgi_profile, hookup, posting):
+def restapi(hostname, port, config_dir, debug, wsgi_profile, posting):
     """
     Run the AiiDA REST API server.
 
@@ -55,13 +54,19 @@ def restapi(hostname, port, config_dir, debug, wsgi_profile, hookup, posting):
         verdi -p <profile_name> restapi --hostname 127.0.0.5 --port 6789
     """
     from aiida.restapi.run_api import run_api
+
     # Invoke the runner
-    run_api(
-        hostname=hostname,
-        port=port,
-        config=config_dir,
-        debug=debug,
-        wsgi_profile=wsgi_profile,
-        hookup=hookup,
-        posting=posting,
-    )
+    try:
+        run_api(
+            hostname=hostname,
+            port=port,
+            config=config_dir,
+            debug=debug,
+            wsgi_profile=wsgi_profile,
+            posting=posting,
+        )
+    except ImportError as exc:
+        raise ImportError(
+            'Failed to import modules required for the REST API. '
+            'You may need to install the `rest` extra, e.g. via `pip install aiida-core[rest]`.'
+        ) from exc
