@@ -109,6 +109,23 @@ MPI_PROCS_PER_MACHINE = OverridableOption(
     'Use 0 to specify no default value.',
 )
 
+DEFAULT_MEMORY_PER_MACHINE = OverridableOption(
+    '--default-memory-per-machine',
+    prompt='Default amount of memory per machine (kB).',
+    cls=InteractiveOption,
+    type=click.INT,
+    help='The default amount of RAM (kB) that should be allocated per machine (node), if not otherwise specified.'
+)
+
+USE_DOUBLE_QUOTES = OverridableOption(
+    '--use-double-quotes/--not-use-double-quotes',
+    default=False,
+    cls=InteractiveOption,
+    prompt='Escape CLI arguments in double quotes',
+    help='Whether the command line arguments before and after the executable in the submission script should be '
+    'escaped with single or double quotes.'
+)
+
 PREPEND_TEXT = OverridableOption(
     '--prepend-text',
     cls=TemplateInteractiveOption,
