@@ -14,9 +14,8 @@ import hashlib
 
 import click
 
-from aiida.backends import BACKEND_DJANGO
 from aiida.cmdline.params import options, types
-from aiida.manage.configuration import get_config, get_config_option, Profile
+from aiida.manage.configuration import Profile, get_config, get_config_option
 from aiida.manage.external.postgres import DEFAULT_DBINFO
 from aiida.manage.external.rmq import BROKER_DEFAULTS
 
@@ -42,6 +41,7 @@ def get_profile_attribute_default(attribute_tuple, ctx):
     :return: profile attribute default value if set, or None
     """
     attribute, default = attribute_tuple
+    parts = attribute.split('.')
 
     try:
         validate_profile_parameter(ctx)
@@ -49,7 +49,10 @@ def get_profile_attribute_default(attribute_tuple, ctx):
         return default
     else:
         try:
-            return getattr(ctx.params['profile'], attribute)
+            data = ctx.params['profile'].dictionary
+            for part in parts:
+                data = data[part]
+            return data
         except KeyError:
             return default
 
@@ -61,6 +64,7 @@ def get_repository_uri_default(ctx):
     :return: default repository URI
     """
     import os
+
     from aiida.manage.configuration.settings import AIIDA_CONFIG_FOLDER
 
     validate_profile_parameter(ctx)
@@ -139,8 +143,8 @@ def get_quicksetup_password(ctx, param, value):  # pylint: disable=unused-argume
     config = get_config()
 
     for available_profile in config.profiles:
-        if available_profile.database_username == username:
-            value = available_profile.database_password
+        if available_profile.storage_config['database_username'] == username:
+            value = available_profile.storage_config['database_password']
             break
     else:
         value = get_random_string(16)
@@ -247,81 +251,97 @@ QUICKSETUP_REPOSITORY_URI = options.REPOSITORY_PATH.clone(
 
 SETUP_DATABASE_ENGINE = QUICKSETUP_DATABASE_ENGINE.clone(
     prompt='Database engine',
-    contextual_default=functools.partial(get_profile_attribute_default, ('database_engine', 'postgresql_psycopg2')),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('storage.config.database_engine', 'postgresql_psycopg2')
+    ),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_DATABASE_BACKEND = QUICKSETUP_DATABASE_BACKEND.clone(
     prompt='Database backend',
-    contextual_default=functools.partial(get_profile_attribute_default, ('database_backend', BACKEND_DJANGO)),
+    contextual_default=functools.partial(get_profile_attribute_default, ('storage_backend', 'psql_dos')),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_DATABASE_HOSTNAME = QUICKSETUP_DATABASE_HOSTNAME.clone(
     prompt='Database host',
-    contextual_default=functools.partial(get_profile_attribute_default, ('database_hostname', 'localhost')),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('storage.config.database_hostname', 'localhost')
+    ),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_DATABASE_PORT = QUICKSETUP_DATABASE_PORT.clone(
     prompt='Database port',
-    contextual_default=functools.partial(get_profile_attribute_default, ('database_port', DEFAULT_DBINFO['port'])),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('storage.config.database_port', DEFAULT_DBINFO['port'])
+    ),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_DATABASE_NAME = QUICKSETUP_DATABASE_NAME.clone(
     prompt='Database name',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('database_name', None)),
+    contextual_default=functools.partial(get_profile_attribute_default, ('storage.config.database_name', None)),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_DATABASE_USERNAME = QUICKSETUP_DATABASE_USERNAME.clone(
     prompt='Database username',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('database_username', None)),
+    contextual_default=functools.partial(get_profile_attribute_default, ('storage.config.database_username', None)),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_DATABASE_PASSWORD = QUICKSETUP_DATABASE_PASSWORD.clone(
     prompt='Database password',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('database_password', None)),
+    contextual_default=functools.partial(get_profile_attribute_default, ('storage.config.database_password', None)),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_BROKER_PROTOCOL = QUICKSETUP_BROKER_PROTOCOL.clone(
     prompt='Broker protocol',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('broker_protocol', BROKER_DEFAULTS.protocol)),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('process_control.config.broker_protocol', BROKER_DEFAULTS.protocol)
+    ),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_BROKER_USERNAME = QUICKSETUP_BROKER_USERNAME.clone(
     prompt='Broker username',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('broker_username', BROKER_DEFAULTS.username)),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('process_control.config.broker_username', BROKER_DEFAULTS.username)
+    ),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_BROKER_PASSWORD = QUICKSETUP_BROKER_PASSWORD.clone(
     prompt='Broker password',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('broker_password', BROKER_DEFAULTS.password)),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('process_control.config.broker_password', BROKER_DEFAULTS.password)
+    ),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_BROKER_HOST = QUICKSETUP_BROKER_HOST.clone(
     prompt='Broker host',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('broker_host', BROKER_DEFAULTS.host)),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('process_control.config.broker_host', BROKER_DEFAULTS.host)
+    ),
     cls=options.interactive.InteractiveOption
 )
 
 SETUP_BROKER_PORT = QUICKSETUP_BROKER_PORT.clone(
     prompt='Broker port',
     required=True,
-    contextual_default=functools.partial(get_profile_attribute_default, ('broker_port', BROKER_DEFAULTS.port)),
+    contextual_default=functools.partial(
+        get_profile_attribute_default, ('process_control.config.broker_port', BROKER_DEFAULTS.port)
+    ),
     cls=options.interactive.InteractiveOption
 )
 
@@ -329,7 +349,7 @@ SETUP_BROKER_VIRTUAL_HOST = QUICKSETUP_BROKER_VIRTUAL_HOST.clone(
     prompt='Broker virtual host name',
     required=True,
     contextual_default=functools.partial(
-        get_profile_attribute_default, ('broker_virtual_host', BROKER_DEFAULTS.virtual_host)
+        get_profile_attribute_default, ('process_control.config.broker_virtual_host', BROKER_DEFAULTS.virtual_host)
     ),
     cls=options.interactive.InteractiveOption
 )
@@ -340,3 +360,9 @@ SETUP_REPOSITORY_URI = QUICKSETUP_REPOSITORY_URI.clone(
     contextual_default=get_repository_uri_default,
     cls=options.interactive.InteractiveOption
 )
+
+SETUP_TEST_PROFILE = options.OverridableOption(
+    '--test-profile', is_flag=True, help='Designate the profile to be used for running the test suite only.'
+)
+
+QUICKSETUP_TEST_PROFILE = SETUP_TEST_PROFILE.clone()
