@@ -37,15 +37,14 @@ def load_backend_if_not_loaded():
     If no profile has been loaded yet, the default profile will be loaded first. A spinner will be shown during both
     actions to indicate that the function is working and has not crashed, since loading can take a second.
     """
-    from aiida.manage.configuration import get_profile, load_profile
-    from aiida.manage.manager import get_manager
+    from aiida.manage import get_manager
 
     manager = get_manager()
 
-    if get_profile() is None or not manager.backend_loaded:
+    if manager.get_profile() is None or not manager.profile_storage_loaded:
         with spinner():
-            load_profile()  # This will load the default profile if no profile has already been loaded
-            manager.get_backend()  # This will load the backend of the loaded profile, if not already loaded
+            manager.load_profile()  # This will load the default profile if no profile has already been loaded
+            manager.get_profile_storage()  # This will load the backend of the loaded profile, if not already loaded
 
 
 def with_dbenv():
@@ -64,11 +63,11 @@ def with_dbenv():
     @decorator
     def wrapper(wrapped, _, args, kwargs):
         """The wrapped function that should be called after having loaded the backend."""
-        from aiida.common.exceptions import ConfigurationError, IntegrityError
+        from aiida.common.exceptions import ConfigurationError, IntegrityError, LockedProfileError
 
         try:
             load_backend_if_not_loaded()
-        except (IntegrityError, ConfigurationError) as exception:
+        except (IntegrityError, ConfigurationError, LockedProfileError) as exception:
             echo.echo_critical(str(exception))
 
         return wrapped(*args, **kwargs)
@@ -191,9 +190,10 @@ def deprecated_command(message):
     @decorator
     def wrapper(wrapped, _, args, kwargs):
         """Echo a deprecation warning before doing anything else."""
-        from aiida.cmdline.utils import templates
         from textwrap import wrap
 
+        from aiida.cmdline.utils import templates
+
         template = templates.env.get_template('deprecated.tpl')
         width = 80
         echo.echo(template.render(msg=wrap(message, width - 4), width=width))
