@@ -14,32 +14,43 @@ from collections.abc import Mapping
 import enum
 import inspect
 import logging
-from uuid import UUID
 import traceback
 from types import TracebackType
 from typing import (
-    Any, cast, Dict, Iterable, Iterator, List, MutableMapping, Optional, Type, Tuple, Union, TYPE_CHECKING
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    MutableMapping,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    cast,
 )
+from uuid import UUID
 
 from aio_pika.exceptions import ConnectionClosed
+from kiwipy.communications import UnroutableError
 import plumpy.exceptions
 import plumpy.futures
-import plumpy.processes
 import plumpy.persistence
-from plumpy.process_states import ProcessState, Finished
-from kiwipy.communications import UnroutableError
+from plumpy.process_states import Finished, ProcessState
+import plumpy.processes
 
 from aiida import orm
-from aiida.orm.utils import serialize
 from aiida.common import exceptions
 from aiida.common.extendeddicts import AttributeDict
 from aiida.common.lang import classproperty, override
 from aiida.common.links import LinkType
 from aiida.common.log import LOG_LEVEL_REPORT
+from aiida.orm.utils import serialize
 
-from .exit_code import ExitCode, ExitCodesNamespace
 from .builder import ProcessBuilder
-from .ports import InputPort, OutputPort, PortNamespace, PORT_NAMESPACE_SEPARATOR
+from .exit_code import ExitCode, ExitCodesNamespace
+from .ports import PORT_NAMESPACE_SEPARATOR, InputPort, OutputPort, PortNamespace
 from .process_spec import ProcessSpec
 
 if TYPE_CHECKING:
@@ -139,7 +150,7 @@ class Process(plumpy.processes.Process):
         from aiida.manage import manager
 
         self._runner = runner if runner is not None else manager.get_manager().get_runner()
-        assert self._runner.communicator is not None, 'communicator not set for runner'
+        # assert self._runner.communicator is not None, 'communicator not set for runner'
 
         super().__init__(
             inputs=self.spec().inputs.serialize(inputs),
@@ -566,13 +577,13 @@ class Process(plumpy.processes.Process):
                 self.node.store_all()
                 if self.node.is_finished_ok:
                     self._state = Finished(self, None, True)  # pylint: disable=attribute-defined-outside-init
-                    for entry in self.node.get_outgoing(link_type=LinkType.RETURN):
+                    for entry in self.node.base.links.get_outgoing(link_type=LinkType.RETURN):
                         if entry.link_label.endswith(f'_{entry.node.pk}'):
                             continue
                         self.out(entry.link_label, entry.node)
                     # This is needed for CalcJob. In that case, the outputs are
                     # returned regardless of whether they end in '_pk'
-                    for entry in self.node.get_outgoing(link_type=LinkType.CREATE):
+                    for entry in self.node.base.links.get_outgoing(link_type=LinkType.CREATE):
                         self.out(entry.link_label, entry.node)
             except exceptions.ModificationNotAllowed:
                 # The calculation was already stored
@@ -619,7 +630,8 @@ class Process(plumpy.processes.Process):
             return
 
         outputs_flat = self._flat_outputs()
-        outputs_stored = self.node.get_outgoing(link_type=(LinkType.CREATE, LinkType.RETURN)).all_link_labels()
+        outputs_stored = self.node.base.links.get_outgoing(link_type=(LinkType.CREATE, LinkType.RETURN)
+                                                           ).all_link_labels()
         outputs_new = set(outputs_flat.keys()) - set(outputs_stored)
 
         for link_label, output in outputs_flat.items():
@@ -628,9 +640,9 @@ class Process(plumpy.processes.Process):
                 continue
 
             if isinstance(self.node, orm.CalculationNode):
-                output.add_incoming(self.node, LinkType.CREATE, link_label)
+                output.base.links.add_incoming(self.node, LinkType.CREATE, link_label)
             elif isinstance(self.node, orm.WorkflowNode):
-                output.add_incoming(self.node, LinkType.RETURN, link_label)
+                output.base.links.add_incoming(self.node, LinkType.RETURN, link_label)
 
             output.store()
 
@@ -663,10 +675,10 @@ class Process(plumpy.processes.Process):
                 raise exceptions.InvalidOperation('calling processes from a calculation type process is forbidden.')
 
             if isinstance(self.node, orm.CalculationNode):
-                self.node.add_incoming(parent_calc, LinkType.CALL_CALC, self.metadata.call_link_label)
+                self.node.base.links.add_incoming(parent_calc, LinkType.CALL_CALC, self.metadata.call_link_label)
 
             elif isinstance(self.node, orm.WorkflowNode):
-                self.node.add_incoming(parent_calc, LinkType.CALL_WORK, self.metadata.call_link_label)
+                self.node.base.links.add_incoming(parent_calc, LinkType.CALL_WORK, self.metadata.call_link_label)
 
         self._setup_metadata()
         self._setup_inputs()
@@ -674,7 +686,7 @@ class Process(plumpy.processes.Process):
     def _setup_metadata(self) -> None:
         """Store the metadata on the ProcessNode."""
         version_info = self.runner.plugin_version_provider.get_version_info(self.__class__)
-        self.node.set_attribute_many(version_info)
+        self.node.base.attributes.set_many(version_info)
 
         for name, metadata in self.metadata.items():
             if name in ['store_provenance', 'dry_run', 'call_link_label']:
@@ -706,10 +718,10 @@ class Process(plumpy.processes.Process):
 
             # Need this special case for tests that use ProcessNodes as classes
             if isinstance(self.node, orm.CalculationNode):
-                self.node.add_incoming(node, LinkType.INPUT_CALC, name)
+                self.node.base.links.add_incoming(node, LinkType.INPUT_CALC, name)
 
             elif isinstance(self.node, orm.WorkflowNode):
-                self.node.add_incoming(node, LinkType.INPUT_WORK, name)
+                self.node.base.links.add_incoming(node, LinkType.INPUT_WORK, name)
 
     def _flat_inputs(self) -> Dict[str, Any]:
         """
@@ -883,7 +895,7 @@ class Process(plumpy.processes.Process):
         # maps the exposed name to all outputs that belong to it
         top_namespace_map = collections.defaultdict(list)
         link_types = (LinkType.CREATE, LinkType.RETURN)
-        process_outputs_dict = node.get_outgoing(link_type=link_types).nested()
+        process_outputs_dict = node.base.links.get_outgoing(link_type=link_types).nested()
 
         for port_name in process_outputs_dict:
             top_namespace = port_name.split(namespace_separator)[0]
@@ -892,6 +904,8 @@ class Process(plumpy.processes.Process):
         for port_namespace in self._get_namespace_list(namespace=namespace, agglomerate=agglomerate):
             # only the top-level key is stored in _exposed_outputs
             for top_name in top_namespace_map:
+                if namespace is not None and namespace not in self.spec()._exposed_outputs:  # pylint: disable=protected-access
+                    raise KeyError(f'the namespace `{namespace}` is not an exposed namespace.')
                 if top_name in self.spec()._exposed_outputs[port_namespace][process_class]:  # pylint: disable=protected-access
                     output_key_map[top_name] = port_namespace
 
@@ -931,12 +945,14 @@ class Process(plumpy.processes.Process):
     def is_valid_cache(cls, node: orm.ProcessNode) -> bool:
         """Check if the given node can be cached from.
 
-        .. warning :: When overriding this method, make sure to call
-            super().is_valid_cache(node) and respect its output. Otherwise,
-            the 'invalidates_cache' keyword on exit codes will not work.
+        Overriding this method allows ``Process`` sub-classes to modify when
+        corresponding process nodes are considered as a cache.
+
+        .. warning :: When overriding this method, make sure to return ``False``
+            *at least* in all cases when ``super()._node.base.caching.is_valid_cache(node)``
+            returns ``False``. Otherwise, the ``invalidates_cache`` keyword on exit
+            codes may have no effect.
 
-        This method allows extending the behavior of `ProcessNode.is_valid_cache`
-        from `Process` sub-classes, for example in plug-ins.
         """
         try:
             return not cls.spec().exit_codes(node.exit_status).invalidates_cache
