@@ -10,9 +10,10 @@
 """
 AiiDA ORM data class storing (numpy) arrays
 """
-
 from ..data import Data
 
+__all__ = ('ArrayData',)
+
 
 class ArrayData(Data):
     """
@@ -44,13 +45,13 @@ class ArrayData(Data):
         :param name: The name of the array to delete from the node.
         """
         fname = f'{name}.npy'
-        if fname not in self.list_object_names():
+        if fname not in self.base.repository.list_object_names():
             raise KeyError(f"Array with name '{name}' not found in node pk= {self.pk}")
 
         # remove both file and attribute
-        self.delete_object(fname)
+        self.base.repository.delete_object(fname)
         try:
-            self.delete_attribute(f'{self.array_prefix}{name}')
+            self.base.attributes.delete(f'{self.array_prefix}{name}')
         except (KeyError, AttributeError):
             # Should not happen, but do not crash if for some reason the property was not set.
             pass
@@ -70,14 +71,14 @@ class ArrayData(Data):
         Return a list of all arrays stored in the node, listing the files (and
         not relying on the properties).
         """
-        return [i[:-4] for i in self.list_object_names() if i.endswith('.npy')]
+        return [i[:-4] for i in self.base.repository.list_object_names() if i.endswith('.npy')]
 
     def _arraynames_from_properties(self):
         """
         Return a list of all arrays stored in the node, listing the attributes
         starting with the correct prefix.
         """
-        return [i[len(self.array_prefix):] for i in self.attributes.keys() if i.startswith(self.array_prefix)]
+        return [i[len(self.array_prefix):] for i in self.base.attributes.keys() if i.startswith(self.array_prefix)]
 
     def get_shape(self, name):
         """
@@ -86,7 +87,7 @@ class ArrayData(Data):
 
         :param name: The name of the array.
         """
-        return tuple(self.get_attribute(f'{self.array_prefix}{name}'))
+        return tuple(self.base.attributes.get(f'{self.array_prefix}{name}'))
 
     def get_iterarrays(self):
         """
@@ -110,11 +111,11 @@ class ArrayData(Data):
             """Return the array stored in a .npy file"""
             filename = f'{name}.npy'
 
-            if filename not in self.list_object_names():
+            if filename not in self.base.repository.list_object_names():
                 raise KeyError(f'Array with name `{name}` not found in ArrayData<{self.pk}>')
 
             # Open a handle in binary read mode as the arrays are written as binary files as well
-            with self.open(filename, mode='rb') as handle:
+            with self.base.repository.open(filename, mode='rb') as handle:
                 return numpy.load(handle, allow_pickle=False)  # pylint: disable=unexpected-keyword-arg
 
         # Return with proper caching if the node is stored, otherwise always re-read from disk
@@ -148,6 +149,7 @@ class ArrayData(Data):
         """
         import re
         import tempfile
+
         import numpy
 
         if not isinstance(array, numpy.ndarray):
@@ -169,10 +171,10 @@ class ArrayData(Data):
             handle.seek(0)
 
             # Write the numpy array to the repository, keeping the byte representation
-            self.put_object_from_filelike(handle, f'{name}.npy', mode='wb', encoding=None)
+            self.base.repository.put_object_from_filelike(handle, f'{name}.npy')
 
         # Store the array name and shape for querying purposes
-        self.set_attribute(f'{self.array_prefix}{name}', list(array.shape))
+        self.base.attributes.set(f'{self.array_prefix}{name}', list(array.shape))
 
     def _validate(self):
         """
@@ -191,3 +193,32 @@ class ArrayData(Data):
                 f'Mismatch of files and properties for ArrayData node (pk= {self.pk}): {files} vs. {properties}'
             )
         super()._validate()
+
+    def _get_array_entries(self):
+        """Return a dictionary with the different array entries.
+
+        The idea is that this dictionary contains the array name as a key and
+        the value is the numpy array transformed into a list. This is so that
+        it can be transformed into a json object.
+        """
+        array_dict = {}
+        for key, val in self.get_iterarrays():
+            array_dict[key] = val.tolist()
+        return array_dict
+
+    def _prepare_json(self, main_file_name='', comments=True):  # pylint: disable=unused-argument
+        """Dump the content of the arrays stored in this node into JSON format.
+
+        :param comments: if True, includes comments (if it makes sense for the given format)
+        """
+        import json
+
+        from aiida import get_file_header
+
+        json_dict = self._get_array_entries()
+        json_dict['original_uuid'] = self.uuid
+
+        if comments:
+            json_dict['comments'] = get_file_header(comment_char='')
+
+        return json.dumps(json_dict).encode('utf-8'), {}
