@@ -15,9 +15,11 @@ functions to operate on them.
 import copy
 import functools
 import itertools
+import json
 
 from aiida.common.constants import elements
 from aiida.common.exceptions import UnsupportedSpeciesError
+
 from .data import Data
 
 __all__ = ('StructureData', 'Kind', 'Site')
@@ -27,8 +29,8 @@ __all__ = ('StructureData', 'Kind', 'Site')
 _MASS_THRESHOLD = 1.e-3
 # Threshold to check if the sum is one or not
 _SUM_THRESHOLD = 1.e-6
-# Threshold used to check if the cell volume is not zero.
-_VOLUME_THRESHOLD = 1.e-6
+# Default cell
+_DEFAULT_CELL = ((0, 0, 0), (0, 0, 0), (0, 0, 0))
 
 _valid_symbols = tuple(i['symbol'] for i in elements.values())
 _atomic_masses = {el['symbol']: el['mass'] for el in elements.values()}
@@ -50,9 +52,6 @@ def _get_valid_cell(inputcell):
     except (IndexError, ValueError, TypeError):
         raise ValueError('Cell must be a list of three vectors, each defined as a list of three coordinates.')
 
-    if abs(calc_cell_volume(the_cell)) < _VOLUME_THRESHOLD:
-        raise ValueError('The cell volume is zero. Invalid cell.')
-
     return the_cell
 
 
@@ -136,25 +135,13 @@ def has_spglib():
 
 def calc_cell_volume(cell):
     """
-    Calculates the volume of a cell given the three lattice vectors.
-
-    It is calculated as cell[0] . (cell[1] x cell[2]), where . represents
-    a dot product and x a cross product.
-
-    :param cell: the cell vectors; the must be a 3x3 list of lists of floats,
-            no other checks are done.
+    Compute the three-dimensional cell volume in Angstrom^3.
 
+    :param cell: the cell vectors; the must be a 3x3 list of lists of floats
     :returns: the cell volume.
     """
-    # pylint: disable=invalid-name
-    # returns the volume of the primitive cell: |a1.(a2xa3)|
-    a1 = cell[0]
-    a2 = cell[1]
-    a3 = cell[2]
-    a_mid_0 = a2[1] * a3[2] - a2[2] * a3[1]
-    a_mid_1 = a2[2] * a3[0] - a2[0] * a3[2]
-    a_mid_2 = a2[0] * a3[1] - a2[1] * a3[0]
-    return abs(a1[0] * a_mid_0 + a1[1] * a_mid_1 + a1[2] * a_mid_2)
+    import numpy as np
+    return np.abs(np.dot(cell[0], np.cross(cell[1], cell[2])))
 
 
 def _create_symbols_tuple(symbols):
@@ -581,6 +568,7 @@ def symop_ortho_from_fract(cell):
     """
     # pylint: disable=invalid-name
     import math
+
     import numpy
 
     a, b, c, alpha, beta, gamma = cell
@@ -605,6 +593,7 @@ def symop_fract_from_ortho(cell):
     """
     # pylint: disable=invalid-name
     import math
+
     import numpy
 
     a, b, c, alpha, beta, gamma = cell
@@ -631,8 +620,8 @@ def ase_refine_cell(aseatoms, **kwargs):
     :return newase: refined cell with reduced set of atoms
     :return symmetry: a dictionary describing the symmetry space group
     """
-    from spglib import refine_cell, get_symmetry_dataset
     from ase.atoms import Atoms
+    from spglib import get_symmetry_dataset, refine_cell
     cell, positions, numbers = refine_cell(aseatoms, **kwargs)
 
     refined_atoms = Atoms(numbers, scaled_positions=positions, cell=cell, pbc=True)
@@ -741,7 +730,7 @@ class StructureData(Data):
 
         super().__init__(**kwargs)
 
-        if any([ext is not None for ext in [ase, pymatgen, pymatgen_structure, pymatgen_molecule]]):
+        if any(ext is not None for ext in [ase, pymatgen, pymatgen_structure, pymatgen_molecule]):
 
             if ase is not None:
                 self.set_ase(ase)
@@ -757,45 +746,23 @@ class StructureData(Data):
 
         else:
             if cell is None:
-                cell = [[1., 0., 0.], [0., 1., 0.], [0., 0., 1.]]
+                cell = _DEFAULT_CELL
+            self.set_cell(cell)
 
             if pbc is None:
                 pbc = [True, True, True]
-
-            self.set_cell(cell)
             self.set_pbc(pbc)
 
     def get_dimensionality(self):
         """
-        This function checks the dimensionality of the structure and
-        calculates its length/surface/volume
-        :return: returns the dimensionality and length/surface/volume
+        Return the dimensionality of the structure and its length/surface/volume.
+
+        Zero-dimensional structures are assigned "volume" 0.
+
+        :return: returns a dictionary with keys "dim" (dimensionality integer), "label" (dimensionality label)
+            and "value" (numerical length/surface/volume).
         """
-
-        import numpy as np
-
-        retdict = {}
-
-        cell = np.array(self.cell)
-        pbc = np.array(self.pbc)
-        dim = len(pbc[pbc])
-
-        retdict['dim'] = dim
-        retdict['label'] = self._dimensionality_label[dim]
-
-        if dim == 0:
-            pass
-        elif dim == 1:
-            retdict['value'] = np.linalg.norm(cell[pbc])
-        elif dim == 2:
-            vectors = cell[pbc]
-            retdict['value'] = np.linalg.norm(np.cross(vectors[0], vectors[1]))
-        elif dim == 3:
-            retdict['value'] = np.dot(cell[0], np.cross(cell[1], cell[2]))
-        else:
-            raise ValueError(f'Dimensionality {dim} must be <= 3')
-
-        return retdict
+        return _get_dimensionality(self.pbc, self.cell)
 
     def set_ase(self, aseatoms):
         """
@@ -944,6 +911,8 @@ class StructureData(Data):
         except ValueError as exc:
             raise ValidationError(f'Invalid periodic boundary conditions: {exc}')
 
+        _validate_dimensionality(self.pbc, self.cell)
+
         try:
             # This will try to create the kinds objects
             kinds = self.kinds
@@ -955,10 +924,7 @@ class StructureData(Data):
         counts = Counter([k.name for k in kinds])
         for count in counts:
             if counts[count] != 1:
-                raise ValidationError(
-                    "Kind with name '{}' appears {} times "
-                    'instead of only one'.format(count, counts[count])
-                )
+                raise ValidationError(f"Kind with name '{count}' appears {counts[count]} times instead of only one")
 
         try:
             # This will try to create the sites objects
@@ -987,7 +953,7 @@ class StructureData(Data):
 
         return_string = 'CRYSTAL\nPRIMVEC 1\n'
         for cell_vector in self.cell:
-            return_string += ' '.join(['%18.10f' % i for i in cell_vector])
+            return_string += ' '.join([f'{i:18.10f}' for i in cell_vector])
             return_string += '\n'
         return_string += 'PRIMCOORD 1\n'
         return_string += f'{int(len(sites))} 1\n'
@@ -1012,16 +978,15 @@ class StructureData(Data):
         Write the given structure to a string of format required by ChemDoodle.
         """
         # pylint: disable=too-many-locals,invalid-name
-        import numpy as np
         from itertools import product
 
-        from aiida.common import json
+        import numpy as np
 
         supercell_factors = [1, 1, 1]
 
         # Get cell vectors and atomic position
-        lattice_vectors = np.array(self.get_attribute('cell'))
-        base_sites = self.get_attribute('sites')
+        lattice_vectors = np.array(self.base.attributes.get('cell'))
+        base_sites = self.base.attributes.get('sites')
 
         start1 = -int(supercell_factors[0] / 2)
         start2 = -int(supercell_factors[1] / 2)
@@ -1124,9 +1089,7 @@ class StructureData(Data):
 
     def _adjust_default_cell(self, vacuum_factor=1.0, vacuum_addition=10.0, pbc=(False, False, False)):
         """
-        If the structure was imported from an xyz file, it lacks a defined cell,
-        and the default cell is taken ([[1,0,0], [0,1,0], [0,0,1]]),
-        leading to an unphysical definition of the structure.
+        If the structure was imported from an xyz file, it lacks a cell.
         This method will adjust the cell
         """
         # pylint: disable=invalid-name
@@ -1138,10 +1101,6 @@ class StructureData(Data):
             """
             return list(zip(*[(min(values), max(values)) for values in zip(*positions)]))
 
-        # First, set PBC
-        # All the checks are done in get_valid_pbc called by set_pbc, no need to check anything here
-        self.set_pbc(pbc)
-
         # Calculating the minimal cell:
         positions = np.array([site.position for site in self.sites])
         position_min, _ = get_extremas_from_positions(positions)
@@ -1149,7 +1108,7 @@ class StructureData(Data):
         # Translate the structure to the origin, such that the minimal values in each dimension
         # amount to (0,0,0)
         positions -= position_min
-        for index, site in enumerate(self.get_attribute('sites')):
+        for index, site in enumerate(self.base.attributes.get('sites')):
             site['position'] = list(positions[index])
 
         # The orthorhombic cell that (just) accomodates the whole structure is now given by the
@@ -1162,6 +1121,11 @@ class StructureData(Data):
         newcell = np.diag(minimal_orthorhombic_cell_dimensions)
         self.set_cell(newcell.tolist())
 
+        # Now set PBC (checks are done in set_pbc, no need to check anything here)
+        self.set_pbc(pbc)
+
+        return self
+
     def get_description(self):
         """
         Returns a string with infos retrieved from StructureData node's properties
@@ -1349,12 +1313,12 @@ class StructureData(Data):
             raise ValueError(f'A kind with the same name ({kind.name}) already exists.')
 
         # If here, no exceptions have been raised, so I add the site.
-        self.attributes.setdefault('kinds', []).append(new_kind.get_raw())
+        self.base.attributes.all.setdefault('kinds', []).append(new_kind.get_raw())
         # Note, this is a dict (with integer keys) so it allows for empty spots!
         if self._internal_kind_tags is None:
             self._internal_kind_tags = {}
 
-        self._internal_kind_tags[len(self.get_attribute('kinds')) - 1] = kind._internal_tag  # pylint: disable=protected-access
+        self._internal_kind_tags[len(self.base.attributes.get('kinds')) - 1] = kind._internal_tag  # pylint: disable=protected-access
 
     def append_site(self, site):
         """
@@ -1373,12 +1337,11 @@ class StructureData(Data):
 
         if site.kind_name not in [kind.name for kind in self.kinds]:
             raise ValueError(
-                "No kind with name '{}', available kinds are: "
-                '{}'.format(site.kind_name, [kind.name for kind in self.kinds])
+                f"No kind with name '{site.kind_name}', available kinds are: {[kind.name for kind in self.kinds]}"
             )
 
         # If here, no exceptions have been raised, so I add the site.
-        self.attributes.setdefault('sites', []).append(new_site.get_raw())
+        self.base.attributes.all.setdefault('sites', []).append(new_site.get_raw())
 
     def append_atom(self, **kwargs):
         """
@@ -1496,7 +1459,7 @@ class StructureData(Data):
         if self.is_stored:
             raise ModificationNotAllowed('The StructureData object cannot be modified, it has already been stored')
 
-        self.set_attribute('kinds', [])
+        self.base.attributes.set('kinds', [])
         self._internal_kind_tags = {}
         self.clear_sites()
 
@@ -1509,7 +1472,7 @@ class StructureData(Data):
         if self.is_stored:
             raise ModificationNotAllowed('The StructureData object cannot be modified, it has already been stored')
 
-        self.set_attribute('sites', [])
+        self.base.attributes.set('sites', [])
 
     @property
     def sites(self):
@@ -1517,7 +1480,7 @@ class StructureData(Data):
         Returns a list of sites.
         """
         try:
-            raw_sites = self.get_attribute('sites')
+            raw_sites = self.base.attributes.get('sites')
         except AttributeError:
             raw_sites = []
         return [Site(raw=i) for i in raw_sites]
@@ -1528,7 +1491,7 @@ class StructureData(Data):
         Returns a list of kinds.
         """
         try:
-            raw_kinds = self.get_attribute('kinds')
+            raw_kinds = self.base.attributes.get('kinds')
         except AttributeError:
             raw_kinds = []
         return [Kind(raw=i) for i in raw_kinds]
@@ -1579,7 +1542,7 @@ class StructureData(Data):
 
         :return: a 3x3 list of lists.
         """
-        return copy.deepcopy(self.get_attribute('cell'))
+        return copy.deepcopy(self.base.attributes.get('cell'))
 
     @cell.setter
     def cell(self, value):
@@ -1594,7 +1557,7 @@ class StructureData(Data):
             raise ModificationNotAllowed('The StructureData object cannot be modified, it has already been stored')
 
         the_cell = _get_valid_cell(value)
-        self.set_attribute('cell', the_cell)
+        self.base.attributes.set('cell', the_cell)
 
     def reset_cell(self, new_cell):
         """
@@ -1610,7 +1573,7 @@ class StructureData(Data):
         if self.is_stored:
             raise ModificationNotAllowed()
 
-        self.set_attribute('cell', new_cell)
+        self.base.attributes.set('cell', new_cell)
 
     def reset_sites_positions(self, new_positions, conserve_particle=True):
         """
@@ -1671,7 +1634,7 @@ class StructureData(Data):
             boundary conditions for the i-th real-space direction (i=1,2,3)
         """
         # return copy.deepcopy(self._pbc)
-        return (self.get_attribute('pbc1'), self.get_attribute('pbc2'), self.get_attribute('pbc3'))
+        return (self.base.attributes.get('pbc1'), self.base.attributes.get('pbc2'), self.base.attributes.get('pbc3'))
 
     @pbc.setter
     def pbc(self, value):
@@ -1687,9 +1650,9 @@ class StructureData(Data):
         the_pbc = get_valid_pbc(value)
 
         # self._pbc = the_pbc
-        self.set_attribute('pbc1', the_pbc[0])
-        self.set_attribute('pbc2', the_pbc[1])
-        self.set_attribute('pbc3', the_pbc[2])
+        self.base.attributes.set('pbc1', the_pbc[0])
+        self.base.attributes.set('pbc2', the_pbc[1])
+        self.base.attributes.set('pbc3', the_pbc[2])
 
     @property
     def cell_lengths(self):
@@ -1754,7 +1717,9 @@ class StructureData(Data):
 
     def get_cell_volume(self):
         """
-        Returns the cell volume in Angstrom^3.
+        Returns the three-dimensional cell volume in Angstrom^3.
+
+        Use the `get_dimensionality` method in order to get the area/length of lower-dimensional cells.
 
         :return: a float.
         """
@@ -1772,9 +1737,10 @@ class StructureData(Data):
             AiiDA database for record. Default False.
         :return: :py:class:`aiida.orm.nodes.data.cif.CifData` node.
         """
-        from .dict import Dict
         from aiida.tools.data import structure as structure_tools
 
+        from .dict import Dict
+
         param = Dict(dict=kwargs)
         try:
             conv_f = getattr(structure_tools, f'_get_cif_{converter}_inline')
@@ -1864,7 +1830,7 @@ class StructureData(Data):
         species = []
         additional_kwargs = {}
 
-        if (kwargs.pop('add_spin', False) and any([n.endswith('1') or n.endswith('2') for n in self.get_kind_names()])):
+        if (kwargs.pop('add_spin', False) and any(n.endswith('1') or n.endswith('2') for n in self.get_kind_names())):
             # case when spins are defined -> no partial occupancy allowed
             from pymatgen.core.periodic_table import Specie
             oxidation_state = 0  # now I always set the oxidation_state to zero
@@ -1884,10 +1850,10 @@ class StructureData(Data):
             for site in self.sites:
                 kind = self.get_kind(site.kind_name)
                 species.append(dict(zip(kind.symbols, kind.weights)))
-            if any([
+            if any(
                 create_automatic_kind_name(self.get_kind(name).symbols,
                                            self.get_kind(name).weights) != name for name in self.get_site_kindnames()
-            ]):
+            ):
                 # add "kind_name" as a properties to each site, whenever
                 # the kind_name cannot be automatically obtained from the symbols
                 additional_kwargs['site_properties'] = {'kind_name': self.get_site_kindnames()}
@@ -2145,18 +2111,14 @@ class Kind:
             return (False, 'Different length of symbols list')
 
         # Check list of symbols
-        for i in range(len(self.symbols)):
-            if self.symbols[i] != other_kind.symbols[i]:
-                return (
-                    False, f'Symbol at position {i + 1:d} are different ({self.symbols[i]} vs. {other_kind.symbols[i]})'
-                )
+        for i, symbol in enumerate(self.symbols):
+            if symbol != other_kind.symbols[i]:
+                return (False, f'Symbol at position {i + 1:d} are different ({symbol} vs. {other_kind.symbols[i]})')
         # Check weights (assuming length of weights and of symbols have same
         # length, which should be always true
-        for i in range(len(self.weights)):
-            if self.weights[i] != other_kind.weights[i]:
-                return (
-                    False, f'Weight at position {i + 1:d} are different ({self.weights[i]} vs. {other_kind.weights[i]})'
-                )
+        for i, weight in enumerate(self.weights):
+            if weight != other_kind.weights[i]:
+                return (False, f'Weight at position {i + 1:d} are different ({weight} vs. {other_kind.weights[i]})')
         # Check masses
         if abs(self.mass - other_kind.mass) > _MASS_THRESHOLD:
             return (False, f'Masses are different ({self.mass} vs. {other_kind.mass})')
@@ -2388,6 +2350,7 @@ class Site:
         """
         # pylint: disable=too-many-branches
         from collections import defaultdict
+
         import ase
 
         # I create the list of tags
@@ -2493,5 +2456,57 @@ class Site:
         return f"kind name '{self.kind_name}' @ {self.position[0]},{self.position[1]},{self.position[2]}"
 
 
-# get_structuredata_from_qeinput has been moved to:
-# aiida.tools.codespecific.quantumespresso.qeinputparser
+def _get_dimensionality(pbc, cell):
+    """
+    Return the dimensionality of the structure and its length/surface/volume.
+
+    Zero-dimensional structures are assigned "volume" 0.
+
+    :return: returns a dictionary with keys "dim" (dimensionality integer), "label" (dimensionality label)
+        and "value" (numerical length/surface/volume).
+    """
+
+    import numpy as np
+
+    retdict = {}
+
+    pbc = np.array(pbc)
+    cell = np.array(cell)
+
+    dim = len(pbc[pbc])
+
+    retdict['dim'] = dim
+    retdict['label'] = StructureData._dimensionality_label[dim]  # pylint: disable=protected-access
+
+    if dim not in (0, 1, 2, 3):
+        raise ValueError(f'Dimensionality {dim} must be one of 0, 1, 2, 3')
+
+    if dim == 0:
+        # We have no concept of 0d volume. Let's return a value of 0 for a consistent output dictionary
+        retdict['value'] = 0
+    elif dim == 1:
+        retdict['value'] = np.linalg.norm(cell[pbc])
+    elif dim == 2:
+        vectors = cell[pbc]
+        retdict['value'] = np.linalg.norm(np.cross(vectors[0], vectors[1]))
+    elif dim == 3:
+        retdict['value'] = calc_cell_volume(cell)
+
+    return retdict
+
+
+def _validate_dimensionality(pbc, cell):
+    """
+    Check whether the given pbc and cell vectors are consistent.
+    """
+    dim = _get_dimensionality(pbc, cell)
+
+    # 0-d structures put no constraints on the cell
+    if dim['dim'] == 0:
+        return
+
+    # finite-d structures should have a cell with finite volume
+    if dim['value'] == 0:
+        raise ValueError(f'Structure has periodicity {pbc} but {dim["dim"]}-d volume 0.')
+
+    return
