@@ -8,18 +8,18 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """Module with `Node` sub class for processes."""
-
 import enum
-from typing import Any, Dict, List, Optional, Tuple, Type, Union
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Type, Union
 
 from plumpy.process_states import ProcessState
 
-from aiida.common.links import LinkType
+from aiida.common import exceptions
 from aiida.common.lang import classproperty
+from aiida.common.links import LinkType
 from aiida.orm.utils.mixins import Sealable
 
-from ..node import Node
+from ..caching import NodeCaching
+from ..node import Node, NodeLinks
 
 if TYPE_CHECKING:
     from aiida.engine.processes import Process
@@ -28,6 +28,98 @@ if TYPE_CHECKING:
 __all__ = ('ProcessNode',)
 
 
+class ProcessNodeCaching(NodeCaching):
+    """Interface to control caching of a node instance."""
+
+    # The link_type might not be correct while the object is being created.
+    _hash_ignored_inputs = ['CALL_CALC', 'CALL_WORK']
+
+    @property
+    def is_valid_cache(self) -> bool:
+        """Return whether the node is valid for caching
+
+        :returns: True if this process node is valid to be used for caching, False otherwise
+        """
+        if not (super().is_valid_cache and self._node.is_finished):
+            return False
+
+        try:
+            process_class = self._node.process_class
+        except ValueError as exc:
+            self._node.logger.warning(
+                f"Not considering {self} for caching, '{exc!r}' when accessing its process class."
+            )
+            return False
+
+        # For process functions, the `process_class` does not have an is_valid_cache attribute
+        try:
+            is_valid_cache_func = process_class.is_valid_cache
+        except AttributeError:
+            return True
+
+        return is_valid_cache_func(self._node)
+
+    @is_valid_cache.setter
+    def is_valid_cache(self, valid: bool) -> None:
+        """Set whether this node instance is considered valid for caching or not.
+
+        :param valid: whether the node is valid or invalid for use in caching.
+        """
+        super().is_valid_cache = valid  # type: ignore[misc]
+
+    def _get_objects_to_hash(self) -> List[Any]:
+        """
+        Return a list of objects which should be included in the hash.
+        """
+        res = super()._get_objects_to_hash()  # pylint: disable=protected-access
+        res.append({
+            entry.link_label: entry.node.base.caching.get_hash()
+            for entry in self._node.base.links.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK))
+            if entry.link_label not in self._hash_ignored_inputs
+        })
+        return res
+
+
+class ProcessNodeLinks(NodeLinks):
+    """Interface for links of a node instance."""
+
+    def validate_incoming(self, source: Node, link_type: LinkType, link_label: str) -> None:
+        """Validate adding a link of the given type from a given node to ourself.
+
+        Adding an input link to a `ProcessNode` once it is stored is illegal because this should be taken care of
+        by the engine in one go. If a link is being added after the node is stored, it is most likely not by the engine
+        and it should not be allowed.
+
+        :param source: the node from which the link is coming
+        :param link_type: the link type
+        :param link_label: the link label
+        :raise TypeError: if `source` is not a Node instance or `link_type` is not a `LinkType` enum
+        :raise ValueError: if the proposed link is invalid
+        """
+        if self._node.is_sealed:
+            raise exceptions.ModificationNotAllowed('Cannot add a link to a sealed node')
+
+        if self._node.is_stored:
+            raise ValueError('attempted to add an input link after the process node was already stored.')
+
+        super().validate_incoming(source, link_type, link_label)
+
+    def validate_outgoing(self, target, link_type, link_label):
+        """Validate adding a link of the given type from ourself to a given node.
+
+        Adding an outgoing link from a sealed node is forbidden.
+
+        :param target: the node to which the link is going
+        :param link_type: the link type
+        :param link_label: the link label
+        :raise aiida.common.ModificationNotAllowed: if the source node (self) is sealed
+        """
+        if self._node.is_sealed:
+            raise exceptions.ModificationNotAllowed('Cannot add a link from a sealed node')
+
+        super().validate_outgoing(target, link_type=link_type, link_label=link_label)
+
+
 class ProcessNode(Sealable, Node):
     """
     Base class for all nodes representing the execution of a process
@@ -40,6 +132,9 @@ class ProcessNode(Sealable, Node):
     """
     # pylint: disable=too-many-public-methods,abstract-method
 
+    _CLS_NODE_LINKS = ProcessNodeLinks
+    _CLS_NODE_CACHING = ProcessNodeCaching
+
     CHECKPOINT_KEY = 'checkpoints'
     EXCEPTION_KEY = 'exception'
     EXIT_MESSAGE_KEY = 'exit_message'
@@ -49,9 +144,6 @@ class ProcessNode(Sealable, Node):
     PROCESS_STATE_KEY = 'process_state'
     PROCESS_STATUS_KEY = 'process_status'
 
-    # The link_type might not be correct while the object is being created.
-    _hash_ignored_inputs = ['CALL_CALC', 'CALL_WORK']
-
     _unstorable_message = 'only Data, WorkflowNode, CalculationNode or their subclasses can be stored'
 
     def __str__(self) -> str:
@@ -95,7 +187,7 @@ class ProcessNode(Sealable, Node):
         :return: `~aiida.engine.processes.builder.ProcessBuilder` instance
         """
         builder = self.process_class.get_builder()
-        builder._update(self.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK)).nested())  # pylint: disable=protected-access
+        builder._update(self.base.links.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK)).nested())  # pylint: disable=protected-access
 
         return builder
 
@@ -107,19 +199,16 @@ class ProcessNode(Sealable, Node):
         :raises ValueError: if no process type is defined, it is an invalid process type string or cannot be resolved
             to load the corresponding class
         """
-        from aiida.common import exceptions
         from aiida.plugins.entry_point import load_entry_point_from_string
 
         if not self.process_type:
-            raise ValueError(f'no process type for CalcJobNode<{self.pk}>: cannot recreate process class')
+            raise ValueError(f'no process type for Node<{self.pk}>: cannot recreate process class')
 
         try:
             process_class = load_entry_point_from_string(self.process_type)
         except exceptions.EntryPointError as exception:
             raise ValueError(
-                'could not load process class for entry point {} for CalcJobNode<{}>: {}'.format(
-                    self.pk, self.process_type, exception
-                )
+                f'could not load process class for entry point `{self.process_type}` for Node<{self.pk}>: {exception}'
             )
         except ValueError:
             try:
@@ -127,9 +216,9 @@ class ProcessNode(Sealable, Node):
                 module_name, class_name = self.process_type.rsplit('.', 1)
                 module = importlib.import_module(module_name)
                 process_class = getattr(module, class_name)
-            except (ValueError, ImportError):
+            except (AttributeError, ValueError, ImportError) as exception:
                 raise ValueError(
-                    f'could not load process class CalcJobNode<{self.pk}> given its `process_type`: {self.process_type}'
+                    f'could not load process class from `{self.process_type}` for Node<{self.pk}>: {exception}'
                 )
 
         return process_class
@@ -149,7 +238,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: the process label
         """
-        return self.get_attribute(self.PROCESS_LABEL_KEY, None)
+        return self.base.attributes.get(self.PROCESS_LABEL_KEY, None)
 
     def set_process_label(self, label: str) -> None:
         """
@@ -157,7 +246,7 @@ class ProcessNode(Sealable, Node):
 
         :param label: process label string
         """
-        self.set_attribute(self.PROCESS_LABEL_KEY, label)
+        self.base.attributes.set(self.PROCESS_LABEL_KEY, label)
 
     @property
     def process_state(self) -> Optional[ProcessState]:
@@ -166,7 +255,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: the process state instance of ProcessState enum
         """
-        state = self.get_attribute(self.PROCESS_STATE_KEY, None)
+        state = self.base.attributes.get(self.PROCESS_STATE_KEY, None)
 
         if state is None:
             return state
@@ -181,7 +270,7 @@ class ProcessNode(Sealable, Node):
         """
         if isinstance(state, ProcessState):
             state = state.value
-        return self.set_attribute(self.PROCESS_STATE_KEY, state)
+        return self.base.attributes.set(self.PROCESS_STATE_KEY, state)
 
     @property
     def process_status(self) -> Optional[str]:
@@ -192,7 +281,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: the process status
         """
-        return self.get_attribute(self.PROCESS_STATUS_KEY, None)
+        return self.base.attributes.get(self.PROCESS_STATUS_KEY, None)
 
     def set_process_status(self, status: Optional[str]) -> None:
         """
@@ -205,7 +294,7 @@ class ProcessNode(Sealable, Node):
         """
         if status is None:
             try:
-                self.delete_attribute(self.PROCESS_STATUS_KEY)
+                self.base.attributes.delete(self.PROCESS_STATUS_KEY)
             except AttributeError:
                 pass
             return None
@@ -213,7 +302,7 @@ class ProcessNode(Sealable, Node):
         if not isinstance(status, str):
             raise TypeError('process status should be a string')
 
-        return self.set_attribute(self.PROCESS_STATUS_KEY, status)
+        return self.base.attributes.set(self.PROCESS_STATUS_KEY, status)
 
     @property
     def is_terminated(self) -> bool:
@@ -295,7 +384,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: the exit status, an integer exit code or None
         """
-        return self.get_attribute(self.EXIT_STATUS_KEY, None)
+        return self.base.attributes.get(self.EXIT_STATUS_KEY, None)
 
     def set_exit_status(self, status: Union[None, enum.Enum, int]) -> None:
         """
@@ -312,7 +401,7 @@ class ProcessNode(Sealable, Node):
         if not isinstance(status, int):
             raise ValueError(f'exit status has to be an integer, got {status}')
 
-        return self.set_attribute(self.EXIT_STATUS_KEY, status)
+        return self.base.attributes.set(self.EXIT_STATUS_KEY, status)
 
     @property
     def exit_message(self) -> Optional[str]:
@@ -321,7 +410,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: the exit message
         """
-        return self.get_attribute(self.EXIT_MESSAGE_KEY, None)
+        return self.base.attributes.get(self.EXIT_MESSAGE_KEY, None)
 
     def set_exit_message(self, message: Optional[str]) -> None:
         """
@@ -335,7 +424,7 @@ class ProcessNode(Sealable, Node):
         if not isinstance(message, str):
             raise ValueError(f'exit message has to be a string type, got {type(message)}')
 
-        return self.set_attribute(self.EXIT_MESSAGE_KEY, message)
+        return self.base.attributes.set(self.EXIT_MESSAGE_KEY, message)
 
     @property
     def exception(self) -> Optional[str]:
@@ -347,7 +436,7 @@ class ProcessNode(Sealable, Node):
         :returns: the exception message or None
         """
         if self.is_excepted:
-            return self.get_attribute(self.EXCEPTION_KEY, '')
+            return self.base.attributes.get(self.EXCEPTION_KEY, '')
 
         return None
 
@@ -360,7 +449,7 @@ class ProcessNode(Sealable, Node):
         if not isinstance(exception, str):
             raise ValueError(f'exception message has to be a string type, got {type(exception)}')
 
-        return self.set_attribute(self.EXCEPTION_KEY, exception)
+        return self.base.attributes.set(self.EXCEPTION_KEY, exception)
 
     @property
     def checkpoint(self) -> Optional[Dict[str, Any]]:
@@ -369,7 +458,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: checkpoint bundle if it exists, None otherwise
         """
-        return self.get_attribute(self.CHECKPOINT_KEY, None)
+        return self.base.attributes.get(self.CHECKPOINT_KEY, None)
 
     def set_checkpoint(self, checkpoint: Dict[str, Any]) -> None:
         """
@@ -377,14 +466,14 @@ class ProcessNode(Sealable, Node):
 
         :param state: string representation of the stepper state info
         """
-        return self.set_attribute(self.CHECKPOINT_KEY, checkpoint)
+        return self.base.attributes.set(self.CHECKPOINT_KEY, checkpoint)
 
     def delete_checkpoint(self) -> None:
         """
         Delete the checkpoint bundle set for the process
         """
         try:
-            self.delete_attribute(self.CHECKPOINT_KEY)
+            self.base.attributes.delete(self.CHECKPOINT_KEY)
         except AttributeError:
             pass
 
@@ -395,7 +484,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: True if the Calculation is marked as paused, False otherwise
         """
-        return self.get_attribute(self.PROCESS_PAUSED_KEY, False)
+        return self.base.attributes.get(self.PROCESS_PAUSED_KEY, False)
 
     def pause(self) -> None:
         """
@@ -404,7 +493,7 @@ class ProcessNode(Sealable, Node):
         This serves only to reflect that the corresponding Process is paused and so this method should not be called
         by anyone but the Process instance itself.
         """
-        return self.set_attribute(self.PROCESS_PAUSED_KEY, True)
+        return self.base.attributes.set(self.PROCESS_PAUSED_KEY, True)
 
     def unpause(self) -> None:
         """
@@ -414,7 +503,7 @@ class ProcessNode(Sealable, Node):
         by anyone but the Process instance itself.
         """
         try:
-            self.delete_attribute(self.PROCESS_PAUSED_KEY)
+            self.base.attributes.delete(self.PROCESS_PAUSED_KEY)
         except AttributeError:
             pass
 
@@ -425,7 +514,7 @@ class ProcessNode(Sealable, Node):
 
         :returns: list of process nodes called by this process
         """
-        return self.get_outgoing(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK)).all_nodes()
+        return self.base.links.get_outgoing(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK)).all_nodes()
 
     @property
     def called_descendants(self) -> List['ProcessNode']:
@@ -450,60 +539,8 @@ class ProcessNode(Sealable, Node):
         :returns: process node that called this process node instance or None
         """
         try:
-            caller = self.get_incoming(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK)).one().node
+            caller = self.base.links.get_incoming(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK)).one().node
         except ValueError:
             return None
         else:
             return caller
-
-    def validate_incoming(self, source: Node, link_type: LinkType, link_label: str) -> None:
-        """Validate adding a link of the given type from a given node to ourself.
-
-        Adding an input link to a `ProcessNode` once it is stored is illegal because this should be taken care of
-        by the engine in one go. If a link is being added after the node is stored, it is most likely not by the engine
-        and it should not be allowed.
-
-        :param source: the node from which the link is coming
-        :param link_type: the link type
-        :param link_label: the link label
-        :raise TypeError: if `source` is not a Node instance or `link_type` is not a `LinkType` enum
-        :raise ValueError: if the proposed link is invalid
-        """
-        super().validate_incoming(source, link_type, link_label)
-        if self.is_stored:
-            raise ValueError('attempted to add an input link after the process node was already stored.')
-
-    @property
-    def is_valid_cache(self) -> bool:
-        """
-        Return whether the node is valid for caching
-
-        :returns: True if this process node is valid to be used for caching, False otherwise
-        """
-        if not (super().is_valid_cache and self.is_finished):
-            return False
-        try:
-            process_class = self.process_class
-        except ValueError as exc:
-            self.logger.warning(f"Not considering {self} for caching, '{exc!r}' when accessing its process class.")
-            return False
-        # For process functions, the `process_class` does not have an
-        # is_valid_cache attribute
-        try:
-            is_valid_cache_func = process_class.is_valid_cache
-        except AttributeError:
-            return True
-
-        return is_valid_cache_func(self)
-
-    def _get_objects_to_hash(self) -> List[Any]:
-        """
-        Return a list of objects which should be included in the hash.
-        """
-        res = super()._get_objects_to_hash()
-        res.append({
-            entry.link_label: entry.node.get_hash()
-            for entry in self.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK))
-            if entry.link_label not in self._hash_ignored_inputs
-        })
-        return res
