@@ -8,12 +8,11 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """Mixin classes for ORM classes."""
-
 import inspect
+from typing import List, Optional
 
 from aiida.common import exceptions
-from aiida.common.lang import override
-from aiida.common.lang import classproperty
+from aiida.common.lang import classproperty, override
 
 
 class FunctionCalculationMixin:
@@ -56,7 +55,7 @@ class FunctionCalculationMixin:
         try:
             source_file_path = inspect.getsourcefile(func)
             with open(source_file_path, 'rb') as handle:
-                self.put_object_from_filelike(handle, self.FUNCTION_SOURCE_FILE_PATH, mode='wb', encoding=None)
+                self.base.repository.put_object_from_filelike(handle, self.FUNCTION_SOURCE_FILE_PATH)
         except (IOError, OSError):
             pass
 
@@ -66,14 +65,14 @@ class FunctionCalculationMixin:
 
         :returns: the function name or None
         """
-        return self.get_attribute(self.FUNCTION_NAME_KEY, None)
+        return self.base.attributes.get(self.FUNCTION_NAME_KEY, None)
 
     def _set_function_name(self, function_name):
         """Set the function name of the wrapped function.
 
         :param function_name: the function name
         """
-        self.set_attribute(self.FUNCTION_NAME_KEY, function_name)
+        self.base.attributes.set(self.FUNCTION_NAME_KEY, function_name)
 
     @property
     def function_namespace(self):
@@ -81,14 +80,14 @@ class FunctionCalculationMixin:
 
         :returns: the function namespace or None
         """
-        return self.get_attribute(self.FUNCTION_NAMESPACE_KEY, None)
+        return self.base.attributes.get(self.FUNCTION_NAMESPACE_KEY, None)
 
     def _set_function_namespace(self, function_namespace):
         """Set the function namespace of the wrapped function.
 
         :param function_namespace: the function namespace
         """
-        self.set_attribute(self.FUNCTION_NAMESPACE_KEY, function_namespace)
+        self.base.attributes.set(self.FUNCTION_NAMESPACE_KEY, function_namespace)
 
     @property
     def function_starting_line_number(self):
@@ -96,21 +95,21 @@ class FunctionCalculationMixin:
 
         :returns: the starting line number or None
         """
-        return self.get_attribute(self.FUNCTION_STARTING_LINE_KEY, None)
+        return self.base.attributes.get(self.FUNCTION_STARTING_LINE_KEY, None)
 
     def _set_function_starting_line_number(self, function_starting_line_number):
         """Set the starting line number of the wrapped function in its source file.
 
         :param function_starting_line_number: the starting line number
         """
-        self.set_attribute(self.FUNCTION_STARTING_LINE_KEY, function_starting_line_number)
+        self.base.attributes.set(self.FUNCTION_STARTING_LINE_KEY, function_starting_line_number)
 
     def get_function_source_code(self):
         """Return the absolute path to the source file in the repository.
 
         :returns: the absolute path of the source file in the repository, or None if it does not exist
         """
-        return self.get_object_content(self.FUNCTION_SOURCE_FILE_PATH)
+        return self.base.repository.get_object_content(self.FUNCTION_SOURCE_FILE_PATH)
 
 
 class Sealable:
@@ -123,76 +122,32 @@ class Sealable:
     def _updatable_attributes(cls):  # pylint: disable=no-self-argument
         return (cls.SEALED_KEY,)
 
-    def validate_incoming(self, source, link_type, link_label):
-        """Validate adding a link of the given type from a given node to ourself.
-
-        Adding an incoming link to a sealed node is forbidden.
-
-        :param source: the node from which the link is coming
-        :param link_type: the link type
-        :param link_label: the link label
-        :raise aiida.common.ModificationNotAllowed: if the target node (self) is sealed
-        """
-        if self.is_sealed:
-            raise exceptions.ModificationNotAllowed('Cannot add a link to a sealed node')
-
-        super().validate_incoming(source, link_type=link_type, link_label=link_label)
-
-    def validate_outgoing(self, target, link_type, link_label):
-        """Validate adding a link of the given type from ourself to a given node.
-
-        Adding an outgoing link from a sealed node is forbidden.
-
-        :param target: the node to which the link is going
-        :param link_type: the link type
-        :param link_label: the link label
-        :raise aiida.common.ModificationNotAllowed: if the source node (self) is sealed
-        """
-        if self.is_sealed:
-            raise exceptions.ModificationNotAllowed('Cannot add a link from a sealed node')
-
-        super().validate_outgoing(target, link_type=link_type, link_label=link_label)
-
     @property
     def is_sealed(self):
         """Returns whether the node is sealed, i.e. whether the sealed attribute has been set to True."""
-        return self.get_attribute(self.SEALED_KEY, False)
+        return self.base.attributes.get(self.SEALED_KEY, False)
 
     def seal(self):
         """Seal the node by setting the sealed attribute to True."""
         if not self.is_sealed:
-            self.set_attribute(self.SEALED_KEY, True)
+            self.base.attributes.set(self.SEALED_KEY, True)
 
     @override
-    def set_attribute(self, key, value):
-        """Set an attribute to the given value.
+    def _check_mutability_attributes(self, keys: Optional[List[str]] = None) -> None:  # pylint: disable=unused-argument
+        """Check if the entity is mutable and raise an exception if not.
 
-        :param key: name of the attribute
-        :param value: value of the attribute
-        :raise aiida.common.exceptions.ModificationNotAllowed: if the node is already sealed or if the node
-            is already stored and the attribute is not updatable.
+        This is called from `NodeAttributes` methods that modify the attributes.
+
+        :param keys: the keys that will be mutated, or all if None
         """
         if self.is_sealed:
             raise exceptions.ModificationNotAllowed('attributes of a sealed node are immutable')
 
-        if self.is_stored and key not in self._updatable_attributes:  # pylint: disable=unsupported-membership-test
-            raise exceptions.ModificationNotAllowed(f'`{key}` is not an updatable attribute')
-
-        self.backend_entity.set_attribute(key, value)
-
-    @override
-    def delete_attribute(self, key):
-        """Delete an attribute.
-
-        :param key: name of the attribute
-        :raises AttributeError: if the attribute does not exist
-        :raise aiida.common.exceptions.ModificationNotAllowed: if the node is already sealed or if the node
-            is already stored and the attribute is not updatable.
-        """
-        if self.is_sealed:
-            raise exceptions.ModificationNotAllowed('attributes of a sealed node are immutable')
-
-        if self.is_stored and key not in self._updatable_attributes:  # pylint: disable=unsupported-membership-test
-            raise exceptions.ModificationNotAllowed(f'`{key}` is not an updatable attribute')
-
-        self.backend_entity.delete_attribute(key)
+        if self.is_stored:
+            # here we are more lenient than the base class, since we allow the modification of some attributes
+            if keys is None:
+                raise exceptions.ModificationNotAllowed('Cannot bulk modify attributes of a stored+unsealed node')
+            elif any(key not in self._updatable_attributes for key in keys):
+                raise exceptions.ModificationNotAllowed(
+                    f'Cannot modify non-updatable attributes of a stored+unsealed node: {keys}'
+                )
