@@ -78,21 +78,6 @@ class Scheduler(metaclass=abc.ABCMeta):
         if not issubclass(self._job_resource_class, JobResource):
             raise RuntimeError('the class attribute `_job_resource_class` is not a subclass of `JobResource`.')
 
-    @classmethod
-    def get_valid_schedulers(cls):
-        """Return all available scheduler plugins.
-
-        .. deprecated:: 1.3.0
-
-            Will be removed in `2.0.0`, use `aiida.plugins.entry_point.get_entry_point_names` instead
-        """
-        import warnings
-        from aiida.common.warnings import AiidaDeprecationWarning
-        from aiida.plugins.entry_point import get_entry_point_names
-        message = 'method is deprecated, use `aiida.plugins.entry_point.get_entry_point_names` instead'
-        warnings.warn(message, AiidaDeprecationWarning)  # pylint: disable=no-member
-        return get_entry_point_names('aiida.schedulers')
-
     @classmethod
     def get_short_doc(cls):
         """Return the first non-empty line of the class docstring, if available."""
@@ -185,6 +170,24 @@ class Scheduler(metaclass=abc.ABCMeta):
 
         return '\n'.join(script_lines)
 
+    def _get_submit_script_environment_variables(self, template):  # pylint: disable=no-self-use
+        """Return the part of the submit script header that defines environment variables.
+
+        :parameter template: a `aiida.schedulers.datastrutures.JobTemplate` instance.
+        :return: string containing environment variable declarations.
+        """
+        if not isinstance(template.job_environment, dict):
+            raise ValueError('If you provide job_environment, it must be a dictionary')
+
+        lines = ['# ENVIRONMENT VARIABLES BEGIN ###']
+
+        for key, value in template.job_environment.items():
+            lines.append(f'export {key.strip()}={escape_for_bash(value, template.environment_variables_double_quotes)}')
+
+        lines.append('# ENVIRONMENT VARIABLES END ###')
+
+        return '\n'.join(lines)
+
     @abc.abstractmethod
     def _get_submit_script_header(self, job_tmpl):
         """Return the submit script header, using the parameters from the job template.
@@ -203,9 +206,10 @@ class Scheduler(metaclass=abc.ABCMeta):
     def _get_run_line(self, codes_info, codes_run_mode):
         """Return a string with the line to execute a specific code with specific arguments.
 
-        :parameter codes_info: a list of `aiida.common.datastructures.CodeInfo` objects. Each contains the information
-            needed to run the code. I.e. `cmdline_params`, `stdin_name`, `stdout_name`, `stderr_name`, `join_files`. See
-            the documentation of `JobTemplate` and `CodeInfo`.
+        :parameter codes_info: a list of `aiida.scheduler.datastructures.JobTemplateCodeInfo` objects.
+            Each contains the information needed to run the code. I.e. `cmdline_params`, `stdin_name`,
+            `stdout_name`, `stderr_name`, `join_files`. See
+            the documentation of `JobTemplate` and `JobTemplateCodeInfo`.
         :parameter codes_run_mode: instance of `aiida.common.datastructures.CodeRunMode` contains the information on how
             to launch the multiple codes.
         :return: string with format: [executable] [args] {[ < stdin ]} {[ < stdout ]} {[2>&1 | 2> stderr]}
@@ -215,19 +219,28 @@ class Scheduler(metaclass=abc.ABCMeta):
         list_of_runlines = []
 
         for code_info in codes_info:
+            computer_use_double_quotes = code_info.use_double_quotes[0]
+            code_use_double_quotes = code_info.use_double_quotes[1]
+
             command_to_exec_list = []
+            for arg in code_info.prepend_cmdline_params:
+                command_to_exec_list.append(escape_for_bash(arg, use_double_quotes=computer_use_double_quotes))
             for arg in code_info.cmdline_params:
-                command_to_exec_list.append(escape_for_bash(arg))
+                command_to_exec_list.append(escape_for_bash(arg, use_double_quotes=code_use_double_quotes))
             command_to_exec = ' '.join(command_to_exec_list)
 
-            stdin_str = f'< {escape_for_bash(code_info.stdin_name)}' if code_info.stdin_name else ''
-            stdout_str = f'> {escape_for_bash(code_info.stdout_name)}' if code_info.stdout_name else ''
+            escape_stdin_name = escape_for_bash(code_info.stdin_name, use_double_quotes=computer_use_double_quotes)
+            escape_stdout_name = escape_for_bash(code_info.stdout_name, use_double_quotes=computer_use_double_quotes)
+            escape_sterr_name = escape_for_bash(code_info.stderr_name, use_double_quotes=computer_use_double_quotes)
+
+            stdin_str = f'< {escape_stdin_name}' if code_info.stdin_name else ''
+            stdout_str = f'> {escape_stdout_name}' if code_info.stdout_name else ''
 
             join_files = code_info.join_files
             if join_files:
                 stderr_str = '2>&1'
             else:
-                stderr_str = f'2> {escape_for_bash(code_info.stderr_name)}' if code_info.stderr_name else ''
+                stderr_str = f'2> {escape_sterr_name}' if code_info.stderr_name else ''
 
             output_string = f'{command_to_exec} {stdin_str} {stdout_str} {stderr_str}'
 
@@ -290,35 +303,6 @@ class Scheduler(metaclass=abc.ABCMeta):
 
         return detailed_job_info
 
-    def get_detailed_jobinfo(self, jobid):
-        """
-        Return a string with the output of the detailed_jobinfo command.
-
-        .. deprecated:: 1.1.0
-            Will be removed in `v2.0.0`, use :meth:`aiida.schedulers.scheduler.Scheduler.get_detailed_job_info` instead.
-
-        At the moment, the output text is just retrieved
-        and stored for logging purposes, but no parsing is performed.
-
-        :raises: :class:`aiida.common.exceptions.FeatureNotAvailable`
-        """
-        import warnings
-        from aiida.common.warnings import AiidaDeprecationWarning
-        warnings.warn('function is deprecated, use `get_detailed_job_info` instead', AiidaDeprecationWarning)  # pylint: disable=no-member
-
-        command = self._get_detailed_job_info_command(job_id=jobid)  # pylint: disable=assignment-from-no-return
-        with self.transport:
-            retval, stdout, stderr = self.transport.exec_command_wait(command)
-
-        return f"""Detailed jobinfo obtained with command '{command}'
-Return Code: {retval}
--------------------------------------------------------------
-stdout:
-{stdout}
-stderr:
-{stderr}
-"""
-
     @abc.abstractmethod
     def _parse_joblist_output(self, retval, stdout, stderr):
         """Parse the joblist output as returned by executing the command returned by `_get_joblist_command` method.
@@ -415,14 +399,14 @@ stderr:
         :return: True if everything seems ok, False otherwise.
         """
 
-    def parse_output(self, detailed_job_info, stdout, stderr):
+    def parse_output(self, detailed_job_info=None, stdout=None, stderr=None):
         """Parse the output of the scheduler.
 
         :param detailed_job_info: dictionary with the output returned by the `Scheduler.get_detailed_job_info` command.
             This should contain the keys `retval`, `stdout` and `stderr` corresponding to the return value, stdout and
             stderr returned by the accounting command executed for a specific job id.
-        :param stdout: string with the output written by the scheduler to stdout
-        :param stderr: string with the output written by the scheduler to stderr
-        :return: None or an instance of `aiida.engine.processes.exit_code.ExitCode`
+        :param stdout: string with the output written by the scheduler to stdout.
+        :param stderr: string with the output written by the scheduler to stderr.
+        :return: None or an instance of :class:`aiida.engine.processes.exit_code.ExitCode`.
         """
         raise exceptions.FeatureNotAvailable(f'output parsing is not available for `{self.__class__.__name__}`')
