@@ -220,10 +220,11 @@ class DbEntry:
         Returns raw contents of a file as string.
         """
         if self._contents is None:
-            from urllib.request import urlopen
             from hashlib import md5
+            from urllib.request import urlopen
 
-            self._contents = urlopen(self.source['uri']).read().decode('utf-8')
+            with urlopen(self.source['uri']) as handle:
+                self._contents = handle.read().decode('utf-8')
             self.source['source_md5'] = md5(self._contents.encode('utf-8')).hexdigest()
         return self._contents
 
@@ -281,9 +282,10 @@ class CifEntry(DbEntry):
 
         :return: :py:class:`aiida.orm.nodes.data.cif.CifData` object
         """
-        from aiida.orm.nodes.data.cif import CifData
         import tempfile
 
+        from aiida.orm.nodes.data.cif import CifData
+
         cifnode = None
 
         with tempfile.NamedTemporaryFile(mode='w+') as handle:
@@ -326,15 +328,16 @@ class UpfEntry(DbEntry):
 
         :return: :py:class:`aiida.orm.nodes.data.upf.UpfData` object
         """
-        from aiida.orm import UpfData
         import tempfile
 
+        from aiida.orm import UpfData
+
         upfnode = None
 
         # Prefixing with an ID in order to start file name with the name
         # of the described element.
-        with tempfile.NamedTemporaryFile(mode='w+', prefix=self.source['id']) as handle:
-            handle.write(self.contents)
+        with tempfile.NamedTemporaryFile(mode='w+b', prefix=self.source['id']) as handle:
+            handle.write(self.contents.encode('utf-8'))
             handle.flush()
             upfnode = UpfData(file=handle.name, source=self.source)
 
